<?php

/*

    GTPhoto2 - A multi user photo album
    Copyright (C) 2004-2006 Chris Kloosterman

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Author's Email Address:
        kloostec@gavintech.com
    Paper Mail:
        5945 Cassino Road
        Duncan, BC, V9L 4G5, Canada

*/

# this script provides wrappers for all of the database functions

function sql_connect($host, $user, $password, $database = "") {
	if (DATABASE_TYPE == "mysql") {
		$db_link = mysql_connect($host, $user, $password);
		return $db_link;
	} elseif (DATABASE_TYPE == "mysqli") {
		if ($database == "") {
			$db_link = mysqli_connect($host, $user, $password);
		} else {
			$db_link = mysqli_connect($host, $user, $password, $database);
		}
		return $db_link;
	}
}

function sql_select_db($database,$db_link) {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_select_db($database, $db_link));
	} elseif (DATABASE_TYPE == "mysqli") {
		return(mysqli_select_db($db_link,$database));
	}
}

function sql_query($sql) {
	echo ("<p><b>Warning:</b> Deprecated sql_query function used!</p>");
	return sql_query_write($sql);
}

function sql_query_read($sql, $database=DEFAULT_DATABASE) {

	global $db_link_read;
	global $db_link_write;

	# change databases if necessary
	if (DATABASE_TYPE == "mysql") {
		# mysql is buggy, and we need to switch both over
		global $last_database;
		if ($last_database != $database) {
			if (DEBUG) {
				echo ("<p>Changing database to $database</p>");
			}
			sql_select_db($database, $db_link_read);
			sql_select_db($database, $db_link_write);
			$last_database = $database;
		}
	} else {
		global $last_read_database;
		if ($last_read_database != $database) {
			if (DEBUG) {
				echo ("<p>Changing database to $database</p>");
			}
			sql_select_db($database, $db_link_read);
			$last_read_database = $database;
		}
	}
	
	# figure out whether we should use the query cache
	$query_cache = array();
	if (DO_CACHING && !DO_CACHING_OVERRIDE) {
		global $query_cache;
	}
	
	# debug print the SQL statement
	if (DEBUG) {
		echo ("<p>Read SQL Query: ".$sql);
	}
	
	if (DATABASE_TYPE == "mysql") {
		if ($query_cache[$sql]) {
			if (mysql_num_rows($query_cache[$sql]) > 0) {
				# we need to rewind back to the beginning of the result set
				# if there was results, in case we have done fetch_array already
				mysql_data_seek($query_cache[$sql],0);
			}
			if (DEBUG) {
				echo (" <b>cache hit</b></p>");
			}
			global $total_cached_sql_queries;
			$total_cached_sql_queries++;
			return $query_cache[$sql];
		} else {
			if (DEBUG) {
				echo (" <b>cache miss</b></p>");
			}
			$result = mysql_query($sql, $db_link_read);
			if (mysql_error()) {
				puke("Mysql Error: ".mysql_error());
			} else {
				$query_cache[$sql] = $result;
				global $total_sql_queries;
				$total_sql_queries++;
				return $result;
			}
		}
	} elseif (DATABASE_TYPE == "mysqli") {
		if ($query_cache[$sql]) {
			if (mysqli_num_rows($query_cache[$sql]) > 0) {
				mysqli_data_seek($query_cache[$sql],0);
			}
			if (DEBUG) {
				echo (" <b>cache hit</b></p>");
			}
			global $total_cached_sql_queries;
			$total_cached_sql_queries++;
			return $query_cache[$sql];
		} else {
			if (DEBUG) {
				echo (" <b>cache miss</b></p>");
			}
			$result = mysqli_query($db_link_read,$sql);
			if (mysqli_error($db_link_read)) {
				puke("Mysql Error: ".mysqli_error($db_link_read));
			} else {
				$query_cache[$sql] = $result;
				global $total_sql_queries;
				$total_sql_queries++;
				return $result;
			}
		}
	}

}

function sql_query_write($sql, $database=DEFAULT_DATABASE) {

	global $db_link_write;
	global $db_link_read;

	# check to make sure we're not read-only
	if (READ_ONLY_MODE == true) {
		echo ("<p class=\"error\">Read only mode.  Skipping query.</p>");
		return null;
	}

	# change databases if necessary
	if (DATABASE_TYPE == "mysql") {
		# mysql is buggy, and we need to switch both over
		global $last_database;
		if ($last_database != $database) {
			if (DEBUG) {
				echo ("<p>Changing database to $database</p>");
			}
			sql_select_db($database, $db_link_read);
			sql_select_db($database, $db_link_write);
			$last_database = $database;
		}
	} else {
		global $last_write_database;
		if ($last_write_database != $database) {
			if (DEBUG) {
				echo ("<p>Changing database to $database</p>");
			}
			sql_select_db($database, $db_link_write);
			$last_write_database = $database;
		}
	}

	# debug print the SQL statement
	if (DEBUG) {
		echo ("<p><b>WRITE</b> SQL Query: ".$sql);
	}
	
	if (DATABASE_TYPE == "mysql") {
		$result = mysql_query($sql, $db_link_write);
		if (mysql_error()) {
			puke("Mysql Error: ".mysql_error());
		} else {
			global $total_sql_queries;
			$total_sql_queries++;
			return $result;
		}
	} elseif (DATABASE_TYPE == "mysqli") {
		$result = mysqli_query($db_link_write,$sql);
		if (mysqli_error($db_link_write)) {
			puke("Mysql Error: ".mysqli_error($db_link_write));
		} else {
			global $total_sql_queries;
			$total_sql_queries++;
			return $result;
		}
	}

}

function sql_fetch_array($result) {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_fetch_array($result));
	} elseif (DATABASE_TYPE == "mysqli") {
		return(mysqli_fetch_array($result));
	}
}

function sql_fetch_assoc($result) {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_fetch_assoc($result));
	} elseif (DATABASE_TYPE == "mysqli") {
		return(mysqli_fetch_assoc($result));
	}
}

function sql_fetch_row($result) {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_fetch_row($result));
	} elseif (DATABASE_TYPE == "mysqli") {
		return(mysqli_fetch_row($result));
	}
}

function sql_num_rows($result) {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_num_rows($result));
	} elseif (DATABASE_TYPE == "mysqli") {
		return(mysqli_num_rows($result));
	}
}

function sql_insert_id() {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_insert_id());
	} elseif (DATABASE_TYPE == "mysqli") {
		# always going to be a write (insert)
		global $db_link_write;
		return(mysqli_insert_id($db_link_write));
	}
}

function sql_affected_rows($db_link="") {
	if (DATABASE_TYPE == "mysql") {
		return(mysql_affected_rows());
	} elseif (DATABASE_TYPE == "mysqli") {
		global $db_link_write;
		return(mysqli_affected_rows($db_link_write));
	}
}

?>
